﻿using System.Windows.Input;
using Borepin.Base;
using Borepin.Service.Versioning;
using Prism.Navigation;
using Prism.Services;
using Xamarin.Forms;

namespace Borepin.PageModel
{
    public class MainPageModel : PageModelBase
    {
        #region Private Members
        private readonly IVersioningService _VersioningService;
        #endregion

        #region Constructors
        public MainPageModel(INavigationService navigationService, IPageDialogService pageDialogService, IVersioningService versioningService) : base(navigationService, pageDialogService)
        {
            _VersioningService = versioningService;
            CurrentVersion = _VersioningService.CurrentVersion;
            CurrentBuild = _VersioningService.CurrentBuild;

            NavigateCommand = new Command<string>(NavigateCommandExecute);
        }
        #endregion

        #region Fields
        private string _CurrentVersion;
        public string CurrentVersion
        {
            get => _CurrentVersion;
            set => SetProperty(ref _CurrentVersion, value);
        }

        private string _CurrentBuild;
        public string CurrentBuild
        {
            get => _CurrentBuild;
            set => SetProperty(ref _CurrentBuild, value);
        }
        #endregion

        #region Commands
        private ICommand _NavigationCommand;
        public ICommand NavigateCommand 
        {
            get => _NavigationCommand;
            set => SetProperty(ref _NavigationCommand, value);
        }
        public async void NavigateCommandExecute(string view)
        {
            INavigationResult result = await _NavigationService.NavigateAsync($"NavigationPage/{ view }").ConfigureAwait(false);
            if(result.Exception != null)
            {
                Log.Fatal(result.Exception, "Navigating failed");
            }
        }
        #endregion
    }
}
