﻿using Borepin.Service.Storage.Exceptions;
using FabAccessAPI;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;

namespace Borepin.Service.Storage
{
    public class LoginStorageService : ILoginStorageService
    {
        #region Static Members
        const string StorageKey = "ConnectionData";
        #endregion

        #region Private Members
        private readonly ISecretStorageService _SecretStorageService;
        #endregion

        #region Constructors
        public LoginStorageService(ISecretStorageService secretStorageService)
        {
            _SecretStorageService = secretStorageService;
        }
        #endregion

        #region Methods
        public async Task<IList<ConnectionData>> GetList()
        {
            return await _LoadConnectionData().ConfigureAwait(false);
        }

        public async Task Add(ConnectionData connectionData)
        {
            if (connectionData.Host.Host == string.Empty || connectionData.Username == string.Empty)
            {
                throw new InvalidConnectionExceptoin(connectionData);
            }

            IList<ConnectionData> connectionData_List = await _LoadConnectionData().ConfigureAwait(false);
            if(connectionData_List.Contains(connectionData))
            {
                throw new DuplicateConnectionException(connectionData);
            }

            connectionData_List.Add(connectionData);
            await _SaveConnectionData(connectionData_List).ConfigureAwait(false);
        }
        public async Task Remove(ConnectionData connectionData)
        {
            if (connectionData.Host.Host == string.Empty || connectionData.Username == string.Empty)
            {
                throw new InvalidConnectionExceptoin(connectionData);
            }

            List<ConnectionData> connectionData_List = new List<ConnectionData>(await _LoadConnectionData().ConfigureAwait(false));
            if (connectionData_List.Contains(connectionData))
            {
                throw new MissingConnectionException(connectionData);
            }

            connectionData_List.RemoveAll(x => x.Equals(connectionData));
            await _SaveConnectionData(connectionData_List).ConfigureAwait(false);
        }

        public async Task UpdateTimestamp(ConnectionData connectionData)
        {
            if (connectionData.Host.Host == string.Empty || connectionData.Username == string.Empty)
            {
                throw new InvalidConnectionExceptoin(connectionData);
            }

            List<ConnectionData> connectionData_List = new List<ConnectionData>(await _LoadConnectionData().ConfigureAwait(false));
            if (connectionData_List.Contains(connectionData))
            {
                throw new MissingConnectionException(connectionData);
            }

            connectionData.LastTime = DateTime.UtcNow;

            connectionData_List.RemoveAll(x => x.Equals(connectionData));
            connectionData_List.Add(connectionData);
            await _SaveConnectionData(connectionData_List).ConfigureAwait(false);
        }
        #endregion

        #region Private Methods
        private async Task<IList<ConnectionData>> _LoadConnectionData()
        {
            List<ConnectionData> connectionData_List;
            try
            {
                string data = await _SecretStorageService.GetAsync(StorageKey).ConfigureAwait(false);
                if(data != null)
                {
                    connectionData_List = JsonConvert.DeserializeObject<List<ConnectionData>>(data);
                }
                else
                {
                    connectionData_List = new List<ConnectionData>();
                    await _SecretStorageService.SetAsync(StorageKey, JsonConvert.SerializeObject(connectionData_List)).ConfigureAwait(false);
                }
            }
            catch (JsonSerializationException)
            {
                connectionData_List = new List<ConnectionData>();
                await _SecretStorageService.SetAsync(StorageKey, JsonConvert.SerializeObject(connectionData_List)).ConfigureAwait(false);
            }

            return connectionData_List;
        }

        private async Task _SaveConnectionData(IList<ConnectionData> connectionData_List)
        {
            string data = JsonConvert.SerializeObject(connectionData_List);
            await _SecretStorageService.SetAsync(StorageKey, data).ConfigureAwait(false);
        }
        #endregion
    }
}
